"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.build = exports.preBuild = void 0;
const config_1 = require("../config");
const serve_1 = require("../serve");
const transform_1 = require("../transform");
const transpilerConfig_1 = require("../transpilerConfig");
const graphql_zeus_1 = require("graphql-zeus");
const transformHeaders = (headers) => {
    if (!headers) {
        return undefined;
    }
    return Object.keys(headers).map((k) => `${k}:${headers[k]}`);
};
const preBuild = (config) => __awaiter(void 0, void 0, void 0, function* () {
    (0, transform_1.cleanBuild)(config);
    (0, transpilerConfig_1.regenerateTsConfig)(config);
    yield (0, transform_1.generateBasicTypingsFiles)(config);
    if (config.graphql) {
        yield Promise.all(Object.keys(config.graphql).map((k) => __awaiter(void 0, void 0, void 0, function* () {
            const schema = yield graphql_zeus_1.Utils.getFromUrl(config.graphql[k].url, transformHeaders(config.graphql[k].headers));
            yield (0, transform_1.generateTypingsFiles)({ config, schema, name: k });
            return {
                schema,
                name: k,
            };
        })));
    }
});
exports.preBuild = preBuild;
const build = () => __awaiter(void 0, void 0, void 0, function* () {
    const config = (0, config_1.readConfig)('./purplehaze.json');
    yield (0, exports.preBuild)(config);
    const { close } = yield (0, serve_1.initBrowserBundler)({
        config,
    });
    yield (0, transform_1.transformFiles)({ config });
    yield (0, transform_1.copyStaticFiles)(config);
    yield close();
});
exports.build = build;
//# sourceMappingURL=index.js.map