"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initBrowserBundler = void 0;
const ws_1 = __importDefault(require("ws"));
const http_1 = __importDefault(require("http"));
const browserHtml_1 = require("../browserHtml");
const browser_1 = require("../browser");
const net_1 = require("net");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const paths_1 = require("../paths");
const env_1 = require("../loaders/env");
const internals = (config, build) => {
    const ssg = {
        envs: (0, env_1.envLoader)(),
        config,
        host: build ? `http://0.0.0.0:${config.port}` : '.',
    };
    return `const ssg = ${JSON.stringify(ssg, null, 4)}`;
};
const getPort = (port = 80, maxPort = 65535) => {
    if (maxPort < port) {
        return Promise.reject((() => {
            const e = new Error('EPORTSPEC');
            return e;
        })());
    }
    const server = (0, net_1.createServer)();
    return new Promise((resolve, reject) => server
        .once('error', (error) => ++port > maxPort ? reject(error) : server.listen(port))
        .once('listening', () => server.close(() => resolve(port)))
        .listen(port));
};
const initBrowserBundler = ({ config, }) => __awaiter(void 0, void 0, void 0, function* () {
    const ws = new ws_1.default.Server({
        port: config.websocketPort,
        perMessageDeflate: true,
    });
    ws.on('connection', (client) => {
        client.send(JSON.stringify({ type: 'connected' }));
        client.on('message', (e) => {
            ws.clients.forEach((v) => {
                v.send(e);
            });
        });
    });
    const browserBundle = http_1.default.createServer((req, res) => {
        const requestURL = req.url;
        if (requestURL === '/' || !requestURL) {
            res.writeHead(200, { 'content-type': 'text/html' });
            const rendererFile = fs_1.default
                .readFileSync(path_1.default.join(__dirname, './renderer.js'))
                .toString('utf-8')
                .replace('{{WEBSOCKET_PORT}}', config.websocketPort + '');
            res.write((0, browserHtml_1.browserHtml)(rendererFile));
        }
        else {
            const [, , ...fileNames] = requestURL.split('/');
            const fileName = fileNames.join('/');
            if ((fileName === null || fileName === void 0 ? void 0 : fileName.endsWith('.js')) || (fileName === null || fileName === void 0 ? void 0 : fileName.endsWith('.mjs'))) {
                const filePath = (0, paths_1.pathGenerated)(fileName);
                const pathContent = fs_1.default.readFileSync(filePath).toString('utf-8');
                const fContent = [internals(config), pathContent].join('\n');
                res.writeHead(200, {
                    'content-type': 'text/javascript',
                    'Cache-Control': 'max-age=0',
                });
                res.write(fContent);
            }
        }
        res.end();
    });
    const browserBundlePort = yield getPort(config.port + 1);
    browserBundle.listen(browserBundlePort);
    const browser = yield (0, browser_1.runBrowser)(browserBundlePort);
    return {
        browserBundle,
        ws,
        browser,
        close: () => __awaiter(void 0, void 0, void 0, function* () {
            ws.close();
            browserBundle.close();
            yield browser.close();
        }),
    };
});
exports.initBrowserBundler = initBrowserBundler;
//# sourceMappingURL=index.js.map