"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanBuild = exports.copyStaticFiles = exports.copyFile = exports.transformFiles = exports.generateTypingsFiles = exports.generateBasicTypingsFiles = exports.readFiles = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const fn_1 = require("./fn");
const console_1 = require("../console");
const typeFetcher_1 = require("../typeFetcher");
const fsAddons_1 = require("../fsAddons");
const paths_1 = require("../paths");
const markdown_1 = require("./transformers/markdown");
const tsx_1 = require("./transformers/tsx");
const env_1 = require("./transformers/env");
const constants_1 = require("../constants");
const route_1 = require("./transformers/route");
const html_1 = require("./transformers/html");
const getFiles = (dir) => {
    const result = [];
    const files = [dir];
    do {
        const filepath = files.pop();
        const stat = fs_1.default.lstatSync(filepath);
        if (stat.isDirectory()) {
            fs_1.default.readdirSync(filepath).forEach((f) => files.push(path_1.default.join(filepath, f)));
        }
        else if (stat.isFile()) {
            result.push(path_1.default.relative(dir, filepath));
        }
    } while (files.length !== 0);
    return result;
};
const readFiles = (matchFunction) => (p) => __awaiter(void 0, void 0, void 0, function* () {
    var e_1, _a;
    const allFiles = [];
    try {
        for (var _b = __asyncValues(getFiles(p)), _c; _c = yield _b.next(), !_c.done;) {
            const f = _c.value;
            const t = f;
            if (matchFunction(t)) {
                allFiles.push(t);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return allFiles;
});
exports.readFiles = readFiles;
const generateBasicTypingsFiles = (config) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, env_1.envTransformer)(config);
});
exports.generateBasicTypingsFiles = generateBasicTypingsFiles;
const generateTypingsFiles = ({ name, schema, config, }) => __awaiter(void 0, void 0, void 0, function* () {
    const typings = yield (0, fn_1.GenerateGlobalTypings)({
        config,
        schema,
    });
    const ssgPath = (0, paths_1.pathSsg)(config);
    yield (0, fsAddons_1.fileWriteRecuirsiveAsync)(ssgPath(name, 'index.ts'), typings);
});
exports.generateTypingsFiles = generateTypingsFiles;
const transformFiles = ({ config }) => __awaiter(void 0, void 0, void 0, function* () {
    const { end } = (0, console_1.calcTime)('Build time', 'blueBright');
    const mdFiles = yield (0, exports.readFiles)(fsAddons_1.isMd)(config.in);
    if (mdFiles.length > 0) {
        yield (0, markdown_1.transformMarkdownFiles)(config)(mdFiles);
    }
    const tsFiles = yield (0, exports.readFiles)(fsAddons_1.isTSFile)(config.in);
    if (tsFiles.length > 0) {
        yield (0, tsx_1.transformTsx)(config)(tsFiles);
    }
    const jsFiles = yield (0, exports.readFiles)(fsAddons_1.isJSFile)(constants_1.PATH_GENERATED);
    const rf = yield Promise.all(jsFiles.map((f) => __awaiter(void 0, void 0, void 0, function* () {
        return ({
            name: f,
            path: (0, paths_1.pathOut)(config)(f),
            content: yield fs_1.default.promises.readFile((0, paths_1.pathGenerated)(f)),
        });
    })));
    yield Promise.all(rf.map((outFile) => __awaiter(void 0, void 0, void 0, function* () { return (0, fsAddons_1.fileWriteRecuirsiveAsync)(outFile.path, outFile.content); })));
    yield (0, typeFetcher_1.downloadTypings)(config, rf.map((r) => r.content.toString('utf-8')));
    (0, console_1.message)('Sending code to browser', 'yellowBright');
    const routes = yield (0, html_1.generateHtmlFiles)(config)(jsFiles);
    (0, console_1.message)('Generating routes', 'yellowBright');
    yield (0, route_1.transformRoutes)(config, routes);
    (0, console_1.message)('Code render successful', 'greenBright');
    end();
});
exports.transformFiles = transformFiles;
const copyFile = (config) => (relativeFilePath) => __awaiter(void 0, void 0, void 0, function* () {
    const f = yield fs_1.default.promises.readFile((0, paths_1.pathIn)(config)(relativeFilePath));
    yield (0, fsAddons_1.fileWriteRecuirsiveAsync)((0, paths_1.pathOut)(config)(relativeFilePath), f);
});
exports.copyFile = copyFile;
const copyStaticFiles = (config) => __awaiter(void 0, void 0, void 0, function* () {
    const files = getFiles(config.in);
    yield Promise.all(files
        .filter((f) => !(0, fsAddons_1.isDirectory)((0, paths_1.pathIn)(config)(f)))
        .filter(fsAddons_1.isStaticFile)
        .map((f) => (0, exports.copyFile)(config)(f)));
});
exports.copyStaticFiles = copyStaticFiles;
const cleanBuild = (config) => {
    if (fs_1.default.existsSync(config.out)) {
        fs_1.default.rmSync(config.out, { recursive: true });
    }
};
exports.cleanBuild = cleanBuild;
//# sourceMappingURL=index.js.map