"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bundle = exports.sendAndReceiveCode = void 0;
const ws_1 = __importDefault(require("ws"));
const console_1 = require("../console");
const path_1 = __importDefault(require("path"));
const htmlSkeleton_1 = require("./htmlSkeleton");
const PARSE_STACK_MESSAGE_REGEX = new RegExp(/https?:\/\/[^\/]*\/[^\/]*\//gm);
const sendAndReceiveCode = (filePath, config) => new Promise((resolve) => {
    const wsClient = new ws_1.default(`ws://127.0.0.1:${config.websocketPort}`);
    const operationId = Math.random().toString(36);
    wsClient.on('error', (e) => {
        (0, console_1.message)('Sending code to browser', 'yellowBright');
        throw new Error(e.message);
    });
    wsClient.on('message', (e) => {
        const event = JSON.parse(e.toString());
        if (event.operationId === operationId) {
            if (event.type === 'rendered' && event.result) {
                resolve(event.result);
            }
            if (event.type === 'error') {
                (0, console_1.message)(`Unexpected error ocurred in ${filePath}`, 'red');
                const e = JSON.parse(event.error || '{}');
                const stackMessage = e.stack || e.message || '';
                (0, console_1.message)(stackMessage.replace(PARSE_STACK_MESSAGE_REGEX, config.in.endsWith('/') ? config.in : `${config.in}/`), 'red');
                resolve(undefined);
            }
            if (event.type === 'module') {
                resolve(undefined);
            }
        }
    });
    wsClient.on('open', () => {
        wsClient.send(JSON.stringify({
            code: `${Math.random().toString(32)}/${filePath}`,
            type: 'initial',
            react: filePath.endsWith('x'),
            operationId,
            name: filePath,
        }));
    });
});
exports.sendAndReceiveCode = sendAndReceiveCode;
const bundle = ({ css, config, name, }) => __awaiter(void 0, void 0, void 0, function* () {
    const socketResult = yield (0, exports.sendAndReceiveCode)(name, config);
    if (!socketResult) {
        return;
    }
    if (socketResult.pages) {
        return {
            pages: socketResult.pages.map((b) => (Object.assign(Object.assign({}, b), { body: (0, htmlSkeleton_1.HtmlSkeletonStatic)(Object.assign(Object.assign({}, b), { scriptName: path_1.default.join('..', name), cssName: css ? path_1.default.join('..', css) : undefined, hydrate: socketResult.hydrate })) }))),
        };
    }
    return {
        content: (0, htmlSkeleton_1.HtmlSkeletonStatic)(Object.assign(Object.assign({}, socketResult), { body: socketResult.body, head: socketResult.head, scriptName: name, cssName: css, hydrate: socketResult.hydrate })),
    };
});
exports.bundle = bundle;
//# sourceMappingURL=module.js.map