"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasTwinFile = exports.createTwinFile = exports.injectHtmlFile = exports.generateHtmlFiles = void 0;
const console_1 = require("../../console");
const fsAddons_1 = require("../../fsAddons");
const paths_1 = require("../../paths");
const module_1 = require("../module");
const fs_1 = __importDefault(require("fs"));
const generateHtmlFiles = (config) => (jsFiles) => __awaiter(void 0, void 0, void 0, function* () {
    const htmlFiles = (yield Promise.all(jsFiles.map((f) => (0, exports.injectHtmlFile)({ fileToTransform: f, config })))).filter((f) => f.code);
    (0, console_1.message)(`Writing out ${htmlFiles.length} pages.`, 'yellow');
    const routes = {};
    yield Promise.all(htmlFiles.map(({ name, code }) => __awaiter(void 0, void 0, void 0, function* () {
        if (code === null || code === void 0 ? void 0 : code.pages) {
            yield Promise.all(code.pages.map((page) => __awaiter(void 0, void 0, void 0, function* () {
                const route = [name, page.slug].join('/');
                routes[route] = `/${route}`;
                yield (0, fsAddons_1.fileWriteRecuirsiveAsync)((0, paths_1.pathOut)(config)(name, page.slug + '.html'), page.body);
            })));
            return;
        }
        if (code === null || code === void 0 ? void 0 : code.content) {
            routes[name] = `/${name}`;
            yield (0, fsAddons_1.fileWriteRecuirsiveAsync)(`${(0, paths_1.pathOut)(config)(name)}.html`, code.content);
        }
    })));
    return routes;
});
exports.generateHtmlFiles = generateHtmlFiles;
const injectHtmlFile = ({ fileToTransform, config, }) => __awaiter(void 0, void 0, void 0, function* () {
    const cssFile = (0, exports.hasTwinFile)(fileToTransform, config, 'css');
    return {
        name: (0, exports.createTwinFile)(fileToTransform),
        code: yield (0, module_1.bundle)({
            name: fileToTransform,
            config: config,
            css: cssFile,
        }),
    };
});
exports.injectHtmlFile = injectHtmlFile;
const createTwinFile = (fileToTransform, extension) => {
    const regexResult = fileToTransform.match(fsAddons_1.fileRegex);
    if (!regexResult) {
        throw new Error('Invalid file provided to function. Only accepting files matching mock regex');
    }
    const twinFileName = extension
        ? `${regexResult[1]}.${extension}`
        : regexResult[1];
    return twinFileName;
};
exports.createTwinFile = createTwinFile;
const hasTwinFile = (fileToTransform, config, extension) => {
    let twinFile;
    const twinFileName = (0, exports.createTwinFile)(fileToTransform, extension);
    const hasCorrespondingTwinFile = fs_1.default.existsSync((0, paths_1.pathIn)(config)(twinFileName));
    if (hasCorrespondingTwinFile) {
        twinFile = twinFileName;
    }
    return twinFile;
};
exports.hasTwinFile = hasTwinFile;
//# sourceMappingURL=html.js.map