"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transpileTS = exports.transformTsx = void 0;
const esbuild_1 = require("esbuild");
const console_1 = require("../../console");
const paths_1 = require("../../paths");
const transformTsx = (config) => (tsFiles) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        return Promise.all(tsFiles.map((tsFile) => __awaiter(void 0, void 0, void 0, function* () {
            const isReact = tsFile.endsWith('x');
            const newJsFile = tsFile.replace(/x$/, '').replace(/\.ts$/, '.js');
            const jsFileName = (0, paths_1.pathGenerated)(newJsFile);
            const fileToTranspilePath = (0, paths_1.pathIn)(config)(tsFile);
            yield (0, exports.transpileTS)(fileToTranspilePath, jsFileName, isReact ? 'tsx' : 'ts', './tsconfig.json');
        })));
    }
    catch (error) {
        if (error instanceof Error) {
            (0, console_1.message)(error.message, 'red');
        }
        return;
    }
});
exports.transformTsx = transformTsx;
const transpileTS = (filePath, outfile, loader, options) => {
    return (0, esbuild_1.build)({
        entryPoints: [filePath],
        outfile,
        tsconfig: options,
        loader: {
            '.ts': loader,
            '.tsx': loader,
            '.js': loader,
            '.jsx': loader,
        },
        format: 'esm',
        bundle: true,
        plugins: [
            {
                name: 'resolve-ts-to-js',
                setup(build) {
                    build.onResolve({ filter: /.*/ }, (args) => {
                        if (args.kind === 'entry-point')
                            return;
                        let path = args.path;
                        if (!path.endsWith('.js') && !path.startsWith('http'))
                            path += '.js';
                        return { path, external: true };
                    });
                },
            },
        ],
    });
};
exports.transpileTS = transpileTS;
//# sourceMappingURL=tsx.js.map