"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadTypings = exports.fetchTypings = exports.mergePackages = exports.fetchTypingsForUrl = exports.constructTypingsUrl = exports.constructTypingsDefsUrl = exports.parseDocumentToFindPackages = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const console_1 = require("./console");
const fsAddons_1 = require("./fsAddons");
const transpilerConfig_1 = require("./transpilerConfig");
const paths_1 = require("./paths");
const fs_1 = __importDefault(require("fs"));
const URL_REGEX = new RegExp(/(?:^[ \t]*|await\s)import.*(https:\/\/.*)\/(.*)['|"]/gm);
const parseDocumentToFindPackages = (content) => {
    return [...content.matchAll(URL_REGEX)]
        .filter((m) => m.length > 1)
        .map((m) => ({
        packageName: m[2],
        url: m[1],
    }));
};
exports.parseDocumentToFindPackages = parseDocumentToFindPackages;
const constructTypingsDefsUrl = ({ packageName, baseUrl, }) => `${baseUrl}/@types/${packageName}/index.d.ts`;
exports.constructTypingsDefsUrl = constructTypingsDefsUrl;
const constructTypingsUrl = ({ packageName, baseUrl, typingsPath, }) => `${baseUrl}/${packageName}/${typingsPath}`;
exports.constructTypingsUrl = constructTypingsUrl;
const fetchTypingsForUrl = (url) => __awaiter(void 0, void 0, void 0, function* () {
    const module = yield (0, node_fetch_1.default)(url).then((r) => {
        if (r.status === 404) {
            return;
        }
        return r.text();
    });
    return module;
});
exports.fetchTypingsForUrl = fetchTypingsForUrl;
const mergePackages = (filesContent) => {
    return filesContent
        .map(exports.parseDocumentToFindPackages)
        .reduce((a, b) => {
        b.forEach((p) => {
            if (!a.find((alreadyInArray) => alreadyInArray.packageName === p.packageName)) {
                a.push(p);
            }
        });
        return a;
    }, []);
};
exports.mergePackages = mergePackages;
const fetchTypings = (packages) => __awaiter(void 0, void 0, void 0, function* () {
    if (packages.length === 0) {
        return [];
    }
    (0, console_1.message)('Starting streaming types for packages: ' +
        packages.map((p) => p.packageName).join(', '), 'blueBright');
    const packagesWithTypings = yield Promise.all(packages.map((p) => __awaiter(void 0, void 0, void 0, function* () {
        let lookForTypings = yield (0, exports.fetchTypingsForUrl)((0, exports.constructTypingsUrl)({
            baseUrl: p.url,
            packageName: p.packageName,
            typingsPath: 'types/index.d.ts',
        }));
        lookForTypings =
            lookForTypings ||
                (yield (0, exports.fetchTypingsForUrl)((0, exports.constructTypingsDefsUrl)({
                    baseUrl: p.url,
                    packageName: p.packageName,
                })));
        if (!lookForTypings) {
            (0, console_1.message)(`Can't find typings on "${p.packageName}" Package will remain untyped`, 'redBright');
        }
        return {
            p,
            typings: lookForTypings,
        };
    })));
    (0, console_1.message)('Successfully fetched the types', 'greenBright');
    return packagesWithTypings.filter((p) => p.typings);
});
exports.fetchTypings = fetchTypings;
const downloadTypings = (configFile, filesContent) => __awaiter(void 0, void 0, void 0, function* () {
    const compilerConfig = (0, transpilerConfig_1.getTsConfig)(configFile);
    const packages = (0, exports.mergePackages)(filesContent).filter((p) => {
        var _a, _b;
        const packagePath = `${p.url}/${p.packageName}`;
        const pathExist = !!((_b = (_a = compilerConfig.compilerOptions) === null || _a === void 0 ? void 0 : _a.paths) === null || _b === void 0 ? void 0 : _b[packagePath]);
        if (!pathExist) {
            return true;
        }
        const typingsExist = fs_1.default.existsSync((0, paths_1.pathGeneratedTypings)(p.packageName, 'index.d.ts'));
        if (!typingsExist) {
            return true;
        }
        return false;
    });
    const ts = yield (0, exports.fetchTypings)(packages);
    const paths = {};
    yield Promise.all(ts.map((t) => __awaiter(void 0, void 0, void 0, function* () {
        const typingsPath = (0, paths_1.pathGeneratedTypings)(t.p.packageName, 'index.d.ts');
        (0, console_1.message)(`Installing typings for "${t.p.packageName}"`, 'yellowBright');
        yield (0, fsAddons_1.fileWriteRecuirsiveAsync)(typingsPath, t.typings);
        paths[`${t.p.url}/${t.p.packageName}`] = [typingsPath];
    })));
    (0, transpilerConfig_1.updateTSConfig)(configFile, (tsConfig) => {
        var _a;
        return Object.assign(Object.assign({}, tsConfig), { compilerOptions: Object.assign(Object.assign({}, (tsConfig.compilerOptions || {})), { paths: Object.assign(Object.assign({}, (((_a = tsConfig.compilerOptions) === null || _a === void 0 ? void 0 : _a.paths) || {})), paths) }) });
    });
    return;
});
exports.downloadTypings = downloadTypings;
//# sourceMappingURL=typeFetcher.js.map